;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Graphics routines (TI-84+CE only)                             ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; loadDefaultPalette - load the default game palette
;   input:  none
;   output: none
;------------------------------------------------
loadDefaultPalette:
        ld      hl,palGame
        ld      bc,PALGAME_SIZE
        ; fall through to loadPalette

;------------------------------------------------
; loadPalette - load a palette
;   input:  HL => start of palette data
;           BC = size
;   output: none
;------------------------------------------------
loadPalette:
        ld      de,mpLcdPalette
        ldir
        ret

;------------------------------------------------
; vram8bpp - set up LCD for 8bpp and initialise (pVbuf)
;   input:  none
;   output: none
;------------------------------------------------
vram8bpp:
        ld      a,$27
        ld      (mpLcdCtrl),a                   ; set 8bpp
        ret
        
;------------------------------------------------
; vram16bpp - return LCD to default 16bpp
;   input:  none
;   output: none
;------------------------------------------------
vram16bpp:
        ld      a,$2D
        ld      (mpLcdCtrl),a                   ; set 16bpp  5:6:5
        ret

;------------------------------------------------
; vbufClear - clear current vbuf to black (8bpp)
;   input:  none
;   output: A = COLOUR_BLACK
;           HL => last byte of vbuf
;------------------------------------------------
vbufClear:
        ld      a,COLOUR_BLACK
        ; fall through to vbufFill

;------------------------------------------------
; vbufFill - fill current vbuf (in 8bpp mode)
;   input:  A = byte to fill with
;   output: A = byte that was used
;           HL => last byte of vbuf
;------------------------------------------------
vbufFill:
        ld      hl,(pVbuf)
        push hl \ pop de \ inc de
        ld      (hl),a
        ld      bc,320*240-1
        ldir
        ret

;------------------------------------------------
; vramClear - clear entire vram
;   input:  none
;   output: none
;------------------------------------------------
vramClear:
        ld      a,COLOUR_BLACK
        ; fall through to vramFill

;------------------------------------------------
; vramFill - fill entire vram with a colour
;   input:  A = colour
;   output: none
;------------------------------------------------
vramFill:
        ld      hl,vram
        ld      de,vram+1
        ld      bc,320*240*2-1
        ld      (hl),a
        ldir
        ret

;------------------------------------------------
; vramFlip - flip vram between vbuf1/vbuf2
;   input:  none
;   output: none
;------------------------------------------------
vramFlip:
        ld      de,vbuf1
        ld      hl,(mpLcdBase)
        or      a
        sbc     hl,de
        add     hl,de
        jr      nz,vfSetBuffer
        ld      de,vbuf2
vfSetBuffer:
        ld      (pVbuf),hl
        ld      bc,VBUF_WIN_OFFSET
        add     hl,bc
        ld      (pVbufWin),hl
        ld      (mpLcdBase),de
        ld      hl,mpLcdIcr
        set     2,(hl)
        ld      l,mpLcdRis&$FF
vfWait:
        bit     2,(hl)
        jr      z,vfWait
        ret

;------------------------------------------------
; copyVram - copy vram to vbuf
;   input:  none
;   output: none
;------------------------------------------------
copyVram:
        ld      hl,(pVram)
        ld      de,(pVbuf)
        ld      bc,320*240
        ldir
        ret

;------------------------------------------------
; drawTiles - draw tile background
;   input:  none
;   output: none
;------------------------------------------------
drawTiles:
        push    iy
        ld      hl,(screenX)
        ld      a,l
        HLDIV8()
        ld      (__dtx),hl
        and     %00000111
        neg
        ld      (__dtxpos),a
        
        ld      hl,(screenY)
        ld      a,l
        HLDIV8()
        and     %00000111
        neg
        ld      d,a                             ; y position to start from {-7=>0}
        ld      c,TILE_ROWS+1
dtCol:
        push    hl
        ld      b,TILE_COLS+1
__dtxpos                = $+1
        ld      e,$00                           ; x position to start from (was preset earlier)
        ld      a,(levelWidth)
        ld      h,a
        mlt     hl
        push    de
__dtx                   = $+1
        ld      de,$000000
        add     hl,de
        ex      de,hl
        ld      iy,(tileDataPtr)
        add     iy,de                           ; IY => tile data for row to draw
        pop     de
dtRow:
        push    bc
        push    de
        call    checkE2TriggerIY                ; L = tile to draw
        ld      h,128
        mlt     hl
        add     hl,hl
        ld      bc,startTiles
        add     hl,bc
        call    drawClippedSprite
        inc     iy                              ; IY => next tile in level data
        pop     de
        ld      a,8
        add     a,e
        ld      e,a
        pop     bc
        djnz    dtRow
        pop     hl
        inc     l
        ld      a,8
        add     a,d
        ld      d,a
        dec     c
        jr      nz,dtCol
        pop     iy
        ret

;------------------------------------------------
; drawClippedSprite - draw a 16x16 8bpp sprite to the game window on vbuf (with clipping)
;   input:  HL => sprite
;           E = x pos / 2
;           D = y pos / 2
;   output: none
;------------------------------------------------
drawClippedSprite:
        ; y-clipping
        ld      a,d
        bit     7,a
        jp      z,dcsCheckBottom
        cp      -7
        ret     c
        neg
        ld      c,a
        ld      b,32                            ; 32 bytes of sprite data to skip per row clipped
        mlt     bc
        add     hl,bc
        ld      a,d
        add     a,8
        ld      d,0
        jp      dcsSetRows
dcsCheckBottom:
        cp      (TILE_ROWS*8)-7
        ld      a,8
        jp      c,dcsSetRows
        ld      a,d
        sub     TILE_ROWS*8
        ret     nc
        neg
dcsSetRows:
        add     a,a
        ld      (__dcsRows),a
        ; x-clipping
        ld      a,e
        cp      TILE_COLS*8
        jp      c,dcsCheckRight
        cp      -7
        ret     c
        neg
        add     a,a
        ld bc,0 \ ld c,a                        ; skip 2 bytes per for each column clipped
        add     hl,bc
        ld      a,e
        add     a,8
        ld      e,0
        jp      dcsSetCols
dcsCheckRight:
        cp      (TILE_COLS*8)-7
        ld      a,8
        jp      c,dcsSetCols
        ld      a,e
        sub     TILE_COLS*8
        ret     nc
        neg
dcsSetCols:
        add     a,a
        ld      (__dcsCols),a
        sub     16
        neg
        ld      (__dcsNext),a
        ; prepare to draw
        ex      de,hl                           ; DE => sprite, H = y pos / 2, L = x pos / 2
        sla     h                               ; H = y pos
        ld bc,0 \ ld c,l
        sla     c
        rl      b                               ; BC = x pos
        ld      l,160
        mlt     hl
        add     hl,hl
        add     hl,bc
        ld      bc,(pVbufWin)
        add     hl,bc                           ; HL => location on vbuf
        push    hl
        ex      de,hl                           ; HL => sprite
        pop     ix                              ; IX => vbuf
        ld      bc,0                            ; ensure that BCU is 0
__dcsRows               = $+1
        ld      a,16
dcsRow:
__dcsCols               = $+1
        ld      c,16
        lea     de,ix
        ldir
__dcsNext               = $+1
        ld      c,16
        add     hl,bc
        ld      de,320
        add     ix,de
        dec     a
        jp      nz,dcsRow
        ret

;------------------------------------------------
; drawTransSprite - draw a 16x16 8bpp sprite to the game window on vbuf (with transparency)
;   input:  HL => sprite
;           E = x pos / 2
;           D = y pos / 2
;   output: none
;------------------------------------------------
drawTransSprite:
        ex      de,hl                           ; DE => sprite, H = y pos / 2, L = x pos / 2
        sla     h                               ; H = y pos
        ld bc,0 \ ld c,l
        sla     c
        rl      b                               ; BC = x pos
        ld      l,160
        mlt     hl
        add     hl,hl
        add     hl,bc
        ld      bc,(pVbufWin)
        add     hl,bc                           ; HL => location on vbuf
        push    hl
        ex      de,hl                           ; HL => sprite
        pop     ix                              ; IX => vbuf
__dtsHeight             = $+1
        ld      c,16
dtsRow:
        lea     de,ix
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld a,(hl) \ or a \ jr z,$+3 \ ld (de),a \ inc de \ inc hl
        ld      de,320
        add     ix,de
        dec     c
        jp      nz,dtsRow
        ret

;------------------------------------------------
; drawTransSpriteWhite - draw a 16x16 8bpp sprite to the game window on vbuf, but draws each pixel white (with transparency)
;   input:  HL => sprite
;           E = x pos / 2
;           D = y pos / 2
;   output: none
;------------------------------------------------
drawTransSpriteWhite:
        ex      de,hl                           ; DE => sprite, H = y pos / 2, L = x pos / 2
        sla     h                               ; H = y pos
        ld bc,0 \ ld c,l
        sla     c
        rl      b                               ; BC = x pos
        ld      l,160
        mlt     hl
        add     hl,hl
        add     hl,bc
        ld      bc,(pVbufWin)
        add     hl,bc                           ; HL => location on vbuf
        ld      c,16
dtswRow:
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        ld a,(de) \ or a \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ inc de \ inc hl
        push    de
        ld      de,320-16
        add     hl,de
        pop     de
        dec     c
        jp      nz,dtswRow
        ret

;------------------------------------------------
; drawClippedTransSprite - draw a 16x16 8bpp sprite to the game window on vbuf (with clipping & transparency)
;   input:  HL => sprite
;           E = x pos / 2
;           D = y pos / 2
;   output: none
;------------------------------------------------
drawClippedTransSprite:
        ; y-clipping
        ld      a,d
        bit     7,a
        jp      z,dctsCheckBottom
        cp      -7
        ret     c
        neg
        ld      c,a
        ld      b,32                            ; 32 bytes of sprite data to skip per row clipped
        mlt     bc
        add     hl,bc
        ld      a,d
        add     a,8
        ld      d,0
        jp      dctsSetRows
dctsCheckBottom:
        cp      (TILE_ROWS*8)-7
        ld      a,8
        jp      c,dctsSetRows
        ld      a,d
        sub     TILE_ROWS*8
        ret     nc
        neg
dctsSetRows:
        add     a,a
        ld      (__dctsRows),a
        ; x-clipping
        ld      a,e
        cp      TILE_COLS*8
        jp      c,dctsCheckRight
        cp      -7
        ret     c
        neg
        add     a,a
        ld bc,0 \ ld c,a                        ; skip 2 bytes per for each column clipped
        add     hl,bc
        ld      a,e
        add     a,8
        ld      e,0
        jp      dctsSetCols
dctsCheckRight:
        cp      (TILE_COLS*8)-7
        ld      a,8
        jp      c,dctsSetCols
        ld      a,e
        sub     TILE_COLS*8
        ret     nc
        neg
dctsSetCols:
        ld      (__dctsCols),a
        sub     8
        neg
        add     a,a
        ld      (__dctsNext),a
        ; prepare to draw
        ex      de,hl                           ; DE => sprite, H = y pos / 2, L = x pos / 2
        sla     h                               ; H = y pos
        sla     l                               ; L = x pos
        ld bc,0 \ ld c,l
        ld      l,160
        mlt     hl
        add     hl,hl
        add     hl,bc
        ld      bc,(pVbufWin)
        add     hl,bc                           ; HL => location on vbuf
        push    iy
        push    hl
        ex      de,hl                           ; HL => sprite
        pop     iy                              ; IY => vbuf
        ld      bc,0                            ; ensure that BCU is 0
        push    ix
__dctsRows              = $+2
        ld      ixh,16
dctsRow:
__dctsCols              = $+1
        ld      b,8
        lea     de,iy
dctsCol:
        ld      a,(hl)
        or      a
        jp      z,dctsSkip1
__dctsMod1              = $
        nop \ nop
        ld      (de),a
dctsSkip1:
        inc     de
        inc     hl
        ld      a,(hl)
        or      a
        jp      z,dctsSkip2
__dctsMod2              = $
        nop \ nop
        ld      (de),a
dctsSkip2:
        inc     de
        inc     hl
        djnz    dctsCol
__dctsNext              = $+1
        ld      c,16
        add     hl,bc
        ld      de,320
        add     iy,de
        dec     ixh
        jp      nz,dctsRow
        pop     ix
        pop     iy
        ret

;------------------------------------------------
; drawClippedTransSpriteWhite - same as drawClippedTransSprite, but draws each pixel white (used for boss taking damage)
;   input:  HL => sprite
;           E = x pos / 2
;           D = y pos / 2
;   output: none
;------------------------------------------------
drawClippedTransSpriteWhite:
        ld      a,OP_LD_A_N
        ld      (__dctsMod1),a
        ld      (__dctsMod2),a
        ld      a,COLOUR_WHITE
        ld      (__dctsMod1+1),a
        ld      (__dctsMod2+1),a
        call    drawClippedTransSprite
        xor     a
        ld      (__dctsMod1),a
        ld      (__dctsMod2),a
        ld      (__dctsMod1+1),a
        ld      (__dctsMod2+1),a
        ret

;------------------------------------------------
; drawClippedTransSpriteCloaked - draw a 16x16 8bpp sprite to the game window on vbuf (with clipping, transparency & cloak effect)
;   input:  HL => sprite
;           E = x pos / 2
;           D = y pos / 2
;   output: none
;------------------------------------------------
drawClippedTransSpriteCloaked:
        ; y-clipping
        ld      a,d
        bit     7,a
        jp      z,dctscCheckBottom
        cp      -7
        ret     c
        neg
        ld      c,a
        ld      b,32                            ; 32 bytes of sprite data to skip per row clipped
        mlt     bc
        add     hl,bc
        ld      a,d
        add     a,8
        ld      d,0
        jp      dctscSetRows
dctscCheckBottom:
        cp      (TILE_ROWS*8)-7
        ld      a,8
        jp      c,dctscSetRows
        ld      a,d
        sub     TILE_ROWS*8
        ret     nc
        neg
dctscSetRows:
        add     a,a
        ld      (__dctscRows),a
        ; x-clipping
        ld      a,e
        cp      TILE_COLS*8
        jp      c,dctscCheckRight
        cp      -7
        ret     c
        neg
        add     a,a
        ld bc,0 \ ld c,a                        ; skip 2 bytes per for each column clipped
        add     hl,bc
        ld      a,e
        add     a,8
        ld      e,0
        jp      dctscSetCols
dctscCheckRight:
        cp      (TILE_COLS*8)-7
        ld      a,8
        jp      c,dctscSetCols
        ld      a,e
        sub     TILE_COLS*8
        ret     nc
        neg
dctscSetCols:
        ld      (__dctscCols),a
        sub     8
        neg
        add     a,a
        ld      (__dctscNext),a
        ; prepare to draw
        ex      de,hl                           ; DE => sprite, H = y pos / 2, L = x pos / 2
        sla     h                               ; H = y pos
        sla     l                               ; L = x pos
        ld bc,0 \ ld c,l
        ld      l,160
        mlt     hl
        add     hl,hl
        add     hl,bc
        ld      bc,(pVbufWin)
        add     hl,bc                           ; HL => location on vbuf
        push    iy
        push    hl
        ex      de,hl                           ; HL => sprite
        pop     iy                              ; IY => vbuf
        ld      bc,0                            ; ensure that BCU is 0
        push    ix
__dctscRows             = $+2
        ld      ixh,16
dctscRow:
__dctscCols             = $+1
        ld      b,8
        lea     de,iy
dctscCol:
        ld      a,(hl)
        or      a
        jp      z,dctscSkip1
        inc     de
        ld      a,(de)
        dec     de
        ld      (de),a
dctscSkip1:
        inc     de
        inc     hl
        ld      a,(hl)
        or      a
        jp      z,dctscSkip2
        inc     de
        ld      a,(de)
        dec     de
        ld      (de),a
dctscSkip2:
        inc     de
        inc     hl
        djnz    dctscCol
__dctscNext             = $+1
        ld      c,16
        add     hl,bc
        ld      de,320
        add     iy,de
        dec     ixh
        jp      nz,dctscRow
        pop     ix
        pop     iy
        ret

;------------------------------------------------
; drawImage - draw an image from sbuf to pVbuf
;   input:  DE = x position
;           L = y position
;           B = width / 2
;           C = height
;           (sbuf) = image data
;   output: none
;------------------------------------------------
drawImage:
        ld      h,160
        mlt     hl
        add     hl,hl
        add     hl,de
        ld      de,(pVbuf)
        add     hl,de
        ld      ix,sbuf
        ld      de,320
diRow:
        push    bc
        push    hl
diCol:
        ld      a,(ix)
        ld      (hl),a
        inc     ix
        inc     hl
        ld      a,(ix)
        ld      (hl),a
        inc     ix
        inc     hl
        djnz    diCol
        pop     hl
        add     hl,de
        pop     bc
        dec     c
        jr      nz,diRow
        ret

;------------------------------------------------
; loadLevelGfx - load the correct tileset & palette, reset palShift
;   input:  none
;   output: none
;------------------------------------------------
loadLevelGfx:
        ; always load tileset #0 first, as it includes some tiles that all other tilesets use
        xor     a
        call    loadTileset

        ; bit of a hack, but Episode III tower tilesets always load TCV as a second base tileset
        ld      a,(levelType)
        and     $0F
        cp      6
        jr      c,skipEp3Tiles
        ld      a,5
        call    loadTileset

skipEp3Tiles:
        ; load tileset mods specific to this level
        ld      a,(levelType)
        and     $0F
        call    nz,loadTileset                  ; don't need to reload if tileset #0

        ; load specific door animation pointers
        ld      ix,animSpriteTable
        ld      bc,sprAnimDoorV
        ld      de,sprAnimDoorH
        ld      a,(levelType)
        and     $0F
        cp      4
        jr      c,loadAnimGfx
        ld      bc,sprAnimDoorV3
        ld      de,sprAnimDoorH3
loadAnimGfx:
        ld      (ix+3),bc
        ld      (ix+6),de

        ; load palette mod
        ld      a,(levelType)
        ld      c,a
        and     $0F
        ld      l,a
        ld      h,3
        mlt     hl
        ld      de,palTable
        add     hl,de
        ld      hl,(hl)
        srl     c
        srl     c
        srl     c
        srl     c
        ld      b,3
        mlt     bc
        add     hl,bc
        ld      hl,(hl)
        ld      de,mpLcdPalette+4
        ld      bc,14*2
        ldir
        ld      hl,e2cheatFlags
        bit     invertedLcdCheat,(hl)
        jr      z,resetPalShift
        ld      hl,mpLcdPalette+4
        ld      b,14*2
loadBlearyEyes:
        ld      a,(hl)
        cpl
        ld      (hl),a
        inc     hl
        djnz    loadBlearyEyes

resetPalShift:
        ; reset palShift
        xor     a
        ld      (palShift),a
        ret

;------------------------------------------------
; loadTileset - load all segments for a tileset
;   input:  A = tileset #
;   output: none
;------------------------------------------------
loadTileset:
__gfxHuffman1           = $+1
        ld      hl,$000000
        cp      7
        jr      c,ltStart
__gfxHuffman2           = $+1
        ld      hl,$000000
ltStart:
        ld      (__gfxHuffman),hl
        ld      l,a
        ld      h,3
        mlt     hl
        ld      de,tilesetTable
        add     hl,de
        ld      hl,(hl)                         ; HL => tileset mod data
        ld      b,(hl)                          ; B = # of mods to load
        inc     hl
        ld      c,(hl)                          ; C = first file # to unpack
        inc     hl
ltLoop:
        push    bc
        ld      b,c
        ld      de,(hl)
        inc     hl
        inc     hl
        inc     hl
        push    hl
__gfxHuffman            = $+1
        ld      hl,$000000
        ld      ix,huffTree
        call    HuffExtr
        pop     hl
        pop     bc
        inc     c
        djnz    ltLoop
        ret

;------------------------------------------------
; loadEnemySprites - load the required enemy sprites for this level
;   input:  none
;   output: none
;------------------------------------------------
loadEnemySprites:
        ld      hl,(pDatFile)
        ld      de,enemySprites
        add     hl,de
        
        ; load enemy 1 sprites
        push    hl
        ld      a,(enemy1)
        ld      de,sprEnemy1
        call    lesLoad
        pop     hl
        
        ; load enemy 2 sprites
        ld      a,(enemy2)
        ld      de,sprEnemy2

lesLoad:
        and     %00011111
        push    hl
        push    de
        ld      b,a
        ld      l,a
        ld      h,3
        mlt     hl
        ld      de,enemySpriteTable
        add     hl,de
        pop     de
        ld      (hl),de
        pop     hl
        ld      ix,huffTree
        jp      HuffExtr

;------------------------------------------------
; loadBossSprites - load the required boss sprites for a boss fight
;   input:  (bossType) = boss type
;   output: none
;------------------------------------------------
loadBossSprites:
        ld      a,(bossType)
        ld      hl,(pDatFile)
        ld      de,bossSprites
        add     hl,de
        ld      b,a
        ld      de,sprBoss
        ld      ix,huffTree
        jp      HuffExtr

;------------------------------------------------
; drawWindow - draw a window to vbuf using fillRect to black out and drawRect to draw the border
;   input:  A = border colour
;           E = x pos / 2
;           D = y pos
;           B = width / 2
;           C = height
;   output: none
;------------------------------------------------
drawWindow:
        push    af
        push    bc
        push    de
        ld      a,COLOUR_BLACK
        call    fillRect
        pop     de
        inc d \ inc d
        inc     e
        pop     bc
        dec b \ dec b \ dec b
        dec c \ dec c \ dec c \ dec c
        pop     af
        ; fall through to drawRect

;------------------------------------------------
; drawRect - draw a 2 pixel thick rectangle outline to vbuf
;   input:  A = colour
;           E = x pos / 2
;           D = y pos
;           B = width / 2
;           C = height
;   output: none
;------------------------------------------------
drawRect:
        ; top line
        push    af
        push    bc
        push    de
        call    drawHLine
        pop     de
        pop     bc
        pop     af
        ; bottom line
        push    af
        push    bc
        push    de
        ld      h,a
        ld      a,d
        add     a,c
        sub     2
        ld      d,a
        ld      a,h
        call    drawHLine
        pop     de
        pop     bc
        pop     af
        ; left line
        push    af
        push    bc
        push    de
        call    drawVLine
        pop     de
        pop     bc
        pop     af
        ; right line
        ld      h,a
        ld      a,e
        add     a,b
        ld      e,a
        ld      a,h
        ; fall through to drawVLine

;------------------------------------------------
; drawVLine - draw a 2 pixel thick vertical line to vbuf
;   input:  A = colour
;           E = x pos / 2
;           D = y pos
;           C = height
;   output: none
;------------------------------------------------
drawVLine:
        call    getDrawPtr
        ld      de,319
        ld      b,c
dvlLoop:
        ld      (hl),a
        inc     hl
        ld      (hl),a
        add     hl,de
        djnz    dvlLoop
        ret

;------------------------------------------------
; drawHLine - draw a 2 pixel thick horizontal line to vbuf
;   input:  A = colour
;           E = x pos / 2
;           D = y pos
;           B = width / 2
;   output: none
;------------------------------------------------
drawHLine:
        push    bc
        call    getDrawPtr
        push    hl
        pop     de                              ; DE = vbuf ptr for 1st line
        ld      bc,320
        add     hl,bc                           ; HL = vbuf ptr for 2nd line
        pop     bc
dhlLoop:
        ld (hl),a \ inc hl
        ld (hl),a \ inc hl
        ld (de),a \ inc de
        ld (de),a \ inc de
        djnz    dhlLoop
        ret

;------------------------------------------------
; fillRect - fill a rectangular section of vbuf
;   input:  A = colour
;           E = x pos / 2
;           D = y pos
;           B = width / 2
;           C = height
;   output: none
;------------------------------------------------
fillRect:
        push    af
        call    getDrawPtr
        pop     af
        ld      de,320
frRows:
        push    bc
        push    hl
frCols:
        ld      (hl),a
        inc     hl
        ld      (hl),a
        inc     hl
        djnz    frCols
        pop     hl
        add     hl,de
        pop     bc
        dec     c
        jr      nz,frRows
        ret

;------------------------------------------------
; getDrawPtr - get a vbuf pointer to start drawing lines, etc.
;   input:  E = x pos / 2
;           D = y pos
;   output: HL => location on vbuf
;------------------------------------------------
getDrawPtr:
        ld      h,160
        ld      l,d
        mlt     hl
        add     hl,hl
        ld      d,2
        mlt     de
        add     hl,de
        ld      de,(pVbuf)
        add     hl,de
        ret

;------------------------------------------------
; draw4bppImage - scale up and draw a 160x120 4bpp to vbuf
;   input:  HL => palette
;           (sbuf) = image data
;   output: none
;------------------------------------------------
draw4bppImage:
        ; load palette data
        ld      de,mpLcdPalette+128
        ld      bc,32
        ldir

        ; scale & draw
        ld      hl,sbuf
        ld      de,(pVbuf)
        ; HL => image data, DE => vbuf
        ld      b,120
d4biRow:
        push    bc
        push    de                              ; save ptr to start of vbuf row
        ld      bc,80*256+64
d4biCol:
        ld      a,(hl)
        rrca
        rrca
        rrca
        rrca
        and     $0F
        add     a,c
        ld      (de),a
        inc     de
        ld      (de),a
        inc     de
        ld      a,(hl)
        and     $0F
        add     a,c
        ld      (de),a
        inc     de
        ld      (de),a
        inc     de
        inc     hl
        djnz    d4biCol

        ; duplicate row
        ex      (sp),hl                         ; HL => start of vbuf row, save data ptr
        ld      bc,320
        ldir
        pop     hl                              ; restore image data ptr
        pop     bc                              ; restore row counter
        djnz    d4biRow
        ret

.end
