;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Visual effect routines                                        ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; fadeOut - fade out by decreasing the contrast
;   input:  none
;   output: (contrastBuffer) = original system contrast setting
;------------------------------------------------
fadeOut:
        ld      a,(contrast)
        ld      (contrastBuffer),a              ; save original contrast setting
        ld      b,20                            ; decrement a max of 20 times
fadeOutLoop:
        ld      c,b                             ; C = dec counter
        ld      a,(contrast)
        dec     a
        push    af
        call    setContrast                     ; save new contrast and set it
        pop     af
        or      a                               ; is contrast 0?
        ret     z                               ; if so, leave
        ld      b,6
        call    delay
        ld      b,c                             ; B = dec counter
        djnz    fadeOutLoop                     ; loop
        ret

;------------------------------------------------
; fadeIn - fade in by increasing the contrast back to the original system setting
;   input:  (contrastBuffer) = original system contrast setting
;   output: none
;------------------------------------------------
fadeIn:
        ld      a,(contrast)
        inc     a
        push    af
        call    setContrast                     ; save new contrast and set it
        pop     af
        ld      hl,contrastBuffer               ; HL => value that we're aiming to return to
        cp      (hl)                            ; have we hit the limit?
        ret     z                               ; if so, leave
        ld      b,6
        call    delay
        jr      fadeIn

;------------------------------------------------
; restoreContrast - instantly restore lcd constrast to original system setting
;   input:  (contrastBuffer) = original system setting
;   output: none
;------------------------------------------------
restoreContrast:
        ld      a,(contrastBuffer)              ; A = value to set
        ; fall through to setContrast

;------------------------------------------------
; setContrast - set contrast level
;   input:  A = contrast value (unmasked, etc.)
;   output: none
;------------------------------------------------
setContrast:
        ld      (contrast),a                    ; save new contrast
        add     a,$18
        or      $C0                             ; bit mask
        bcall(_lcdbusy)                         ; wait for lcd
        out     ($10),a                         ; set contrast
        ret

;------------------------------------------------
; fadeOutAndClear - fade out the screen and then restore contrast with a clear screen
;   input:  none
;   output: none
;------------------------------------------------
fadeOutAndClear:
        call    fadeOut
        bcall(_grbufclr)
        bcall(_grbufcpy)
        jr      restoreContrast

;------------------------------------------------
; contrastUp - increase the lcd contrast by 1 step
;   input:  none
;   output: none
;------------------------------------------------
contrastUp:
        ld      a,(contrast)
        cp      $27
        ret     nc                              ; leave if contrast is already at max
        inc     a
        jr      setContrast

;------------------------------------------------
; contrastDown - decrease the lcd contrast by 1 step
;   input:  none
;   output: none
;------------------------------------------------
contrastDown:
        ld      a,(contrast)
        or      a
        ret     z                               ; leave if contrast is already at 0
        dec     a
        jr      setContrast

;------------------------------------------------
; resetZOffset - reset the screen Z-offset
;   input:  none
;   output: none
;------------------------------------------------
resetZOffset:
        ld      a,$40
        call    $000B
        out     ($10),a
        ret

;------------------------------------------------
; invertGbuf - invert full gbuf
;   input:  none
;   output: none
;------------------------------------------------
invertGbuf:
        ld      hl,gbuf
        ld      b,64
        ; fall through to invertLine

;------------------------------------------------
; invertLine - invert lines on gbuf
; invertPartLine - invert part lines on gbuf
;   input:  HL => where to start inverting
;           B = number of gbuf lines to invert
;           C = length of lines in bytes (only for invertPartLine)
;   output: none
;------------------------------------------------
invertLine:
        ld      c,12                            ; C = 12 bytes per gbuf line
invertPartLine:
        ld      de,12                           ; DE = bytes to skip to get to start of next line
invertLineLoop1:
        push    bc
        push    hl
invertLineLoop2:
        ld      a,(hl)                          ; read byte
        cpl                                     ; invert it
        ld      (hl),a                          ; save it
        inc     hl                              ; HL => next byte
        dec     c                               ; dec byte counter
        jr      nz,invertLineLoop2              ; loop until all bytes on line have been inverted
        pop     hl
        pop     bc
        add     hl,de                           ; HL => next line
        djnz    invertLineLoop1                 ; loop until all lines have been inverted
        ret

;------------------------------------------------
; fillLine - fill lines on gbuf
;   input:  HL => where to start filling
;           B = number of gbuf lines to fill
;   output: none
;------------------------------------------------
fillLine:
        ld      c,12                            ; 12 bytes per line
fillLineLoop:
        ld      (hl),$FF                        ; fill byte
        inc     hl                              ; HL => next byte
        dec     c
        jr      nz,fillLineLoop                 ; line isn't done yet
        djnz    fillLine                        ; more lines to do
        ret

;------------------------------------------------
; levelExplode - do multiple explosion animations and then game is over (83P only)
;   input:  none
;   output: none
;   notes:  bcalled from main game loop as ab5_timerZero (to change app pages)
;------------------------------------------------
timerZero:
        pop hl \ pop hl \ pop hl                ; clear bcall from stack
        jr      shakeScreen
levelExplode:
        ; do the level exploding animation, which comprises of the following:
        ; - shake the screen vertically using the lcd driver
        ; - fade out
        ; - fade back in with the level without sprites
        ; - do lots of explosion animations
        ; - go to game over screen
        ld      a,(levelFlags)
        bit     LFLAG_DARKLEVEL,a
        bcallnz(ab5_darkMask)                   ; if a dark level, do the shadow effect
        bcall(_grbufcpy)
shakeScreen:
        ld      b,15                            ; B = shake screen 15 times
shakeScreenLoop:
        push    bc
        ld      a,$42                           ; screen offset of 2
        bcall(_lcdbusy)
        out     ($10),a                         ; send instruction to lcd
        ld      b,6
        call    delay
        ld      a,$40                           ; screen offset of 0
        bcall(_lcdbusy)
        out     ($10),a
        ld      b,6
        call    delay
        ld      a,KG_1
        bcall(ab5_directIn)
        bit     DI_ALPHA,a
        pop     bc
        jp      z,gameOver
        djnz    shakeScreenLoop
        ; neither player is now active
        xor     a
        ld      (p1p),a
        ld      (p2p),a
        call    fadeOut                         ; fade screen out
        ld      b,20
        call    delay                           ; wait a bit
        bcall(ab5_restoreVideo)                 ; restore gbuf without sprites drawn
        ld      a,(levelFlags)
        bit     LFLAG_DARKLEVEL,a
        bcallnz(ab5_darkMask)                   ; if a dark level, do the shadow effect
        bcall(_grbufcpy)
        bcall(ab5_saveVideo)
        call    fadeIn                          ; fade it back in
        call    clearAnimTable                  ; clear animTable
        ld      b,50                            ; B = 50 frames to loop showing explosions
levelExplosionLoop:
        push    bc                              ; save frame counter
        ld      a,$41                           ; A = 1 pixel screen offset
        bit     0,b
        jr      z,levelExplosionShake           ; every 2nd frame only
        ld      a,$40                           ; otherwise no offset this frame
levelExplosionShake:
        out     ($10),a
        bcall(ab5_updateAnims)                  ; update all explosions
        bcall(ab5_randomExplosion)              ; make a new random explosion on screen
        bcall(ab5_saveVideo)                    ; back up gbuf with no explosion sprites
        bcall(ab5_drawAnims)                    ; draw animations
        bcall(_grbufcpy)
        bcall(ab5_restoreVideo)                 ; restore gbuf without sprites
        ld      b,10
        call    delay
        ld      a,KG_1
        bcall(ab5_directIn)
        bit     DI_ALPHA,a
        pop     bc                              ; restore frame counter
        jp      z,gameOver
        djnz    levelExplosionLoop              ; loop
        jp      gameOver

;------------------------------------------------
; clearSplashAnim - clear lcd with the animation used in splashAnim
;   input:  A = $00 or $FF to clear or fill the lcd
;   output: none
;------------------------------------------------
clearSplashAnim:
        ld      hl,(tempFile)
        push hl \ pop de
        inc     de
        ld      (hl),a
        ld      bc,767
        ldir                                    ; clear 768 bytes at start of temp program
        ; fall through to splashAnim

;------------------------------------------------
; splashAnim - animate a splash screen on to the lcd (can be skipped with [ALPHA]
;   input:  (tempFile) = 768 of image data
;   output: CA = [ALPHA] was pressed, otherwise wasn't pressed
;------------------------------------------------
splashAnim:
        ; set up a 128 byte list with range {0-191}
        xor     a                               ; A = 0
        ld      hl,videoBuffer                  ; HL => where to store list
        ld      b,128
splashAnimList:
        ld      (hl),a                          ; save value
        inc     a                               ; inc value
        inc     hl                              ; inc list pointer
        djnz    splashAnimList
        ; prepare to animate
        ld      b,128                           ; B = frame counter (6 bytes of image data to be shown per frame)
splashAnimLoop:
        push    bc
        bcall(ab5_getRandom)                    ; get a random value for A (0 <= A = B)
        ld de,0 \ ld e,a                        ; DE = random value
        ld      hl,videoBuffer
        add     hl,de                           ; HL => list entry nominated by random value
        ld      a,(hl)                          ; A = offset value from list entry
        push hl \ pop de                        ; DE => list entry
        inc     hl                              ; HL => next list entry
        ld      bc,128
        ldir                                    ; remove entry from list by copying all subsequent entries back one byte (will always have garbage at end of list but this matters not)
        ld de,0 \ ld e,a                        ; DE = initial offset
        ld      b,6                             ; B = 6 bytes of image data to show this frame
splashAnimCopy:
        ld      hl,(tempFile)
        add     hl,de
        ld      a,(hl)                          ; A = byte of image data
        ld      hl,gbuf
        add     hl,de
        ld      (hl),a                          ; save image byte to gbuf
        ld      hl,128
        add     hl,de
        ex      de,hl                           ; DE = last offset + 128
        djnz    splashAnimCopy
        bcall(_grbufcpy)
        ld      a,KG_1
        bcall(ab5_directIn)
        bit     DI_ALPHA,a                      ; [ALPHA]?
        pop     bc
        jr      z,splashAnimAlpha               ; if so, skip out
        djnz    splashAnimLoop
        or      a                               ; clear carry ([ALPHA] wasn't pressed)
        ret
splashAnimAlpha:
        scf                                     ; set carry to denote [ALPHA] press
        ret

;------------------------------------------------
; doEpilogue - show the epilogue for this episode
;   input:  none
;   output: none
;------------------------------------------------
doEpilogue:
        call    iniLevelLoad
        ld      bc,0*256+2                      ; B = file #, C = offset to app page with splash data
        ld      hl,(tempFile)
        ld      de,LEVEL_DATA_SIZE-768
        add     hl,de                           ; unpack to end of (tempFile)
        ld      de,splash
        ex      de,hl
        call    ram_huffExtr
        ; get pointers to relevant epilogue data
        ld      hl,(episode)
        ld      a,l
        ld      h,0
        add     hl,hl                           ; HL = episode * 2
        ld      de,epilogueTable
        add     hl,de                           ; HL => ptr to epilogue data
        add     a,2
        ld      c,a                             ; C = offset to app page with epilogue data
        ld      b,0                             ; B = file #0
        LDHLIND()                               ; HL => epilogue data
        ; call routine in ram, which will swap to correct rom page, decompress the epilogue data to ram, and revert to this rom page
        ld      de,(tempFile)
        call    ram_huffExtr
        ; scroll the splash to gbuf
        bcall(_grbufclr)
        ld      b,32                            ; B = 32 frames (2 rows per frame)
        ld      hl,(tempFile)
        ld      de,LEVEL_DATA_SIZE-768
        add     hl,de                           ; HL => splash data
splashScroll:
        push    bc
        push    hl
        ; scroll gbuf up 2 rows
        ld      hl,gbuf+24
        ld      de,gbuf
        ld      bc,768-24
        ldir
        ; copy 24 bytes from temp program to 2nd last row of gbuf
        pop     hl                              ; HL => splash data
        ld      de,gbuf+768-24                  ; DE => 2nd last row of gbuf
        ld      bc,24
        ldir
        push    hl
        bcall(_grbufcpy)                        ; update lcd
        pop     hl
        pop     bc
        djnz    splashScroll
        ld      b,160
        call    delay                           ; hold on splash temporarily
        call    swapBuffers                     ; move gbuf to videoBuffer in preparation for epilogue routine
        bcall(_grbufclr)
        ld      hl,12
        ld      (epScrCnt),hl                   ; set initial counter value (# bytes to scroll)
        ld      hl,gbuf+768-12
        ld      (epScrPtr),hl                   ; set initial pointer value (where to copy from when scrolling)
        ld      hl,(tempFile)                   ; HL => uncompressed epilogue data
epilogueLoop:
        ld      a,(hl)                          ; A = pencol position (or 0 for no text this line)
        or      a                               ; check if text line
        jr      nz,epilogueLine
        inc     hl
        ld      b,(hl)                          ; B = # rows to scroll
        inc     hl                              ; HL => next bit data after this scroll
        push    hl
        push    bc
        bcall(_grbufclr)                        ; clear gbuf to get clear rows
        pop     bc
        jr      epilogueScroll
epilogueLine:
        ld      d,0                             ; D = penrow
        ld      e,a                             ; E = pencol
        ld      (pencol),de
        inc     hl                              ; HL => string
        call    vPutsNoCursor                   ; write text line to gbuf
        push    hl                              ; save pointer to next bit of data
        ld      b,6                             ; B = 6 lines to scroll
epilogueScroll:
        push    bc
        call    swapBuffers
        ; scroll gbuf up one row over the top of previous gbuf contents (excluding top and bottom rows)
        ld      bc,(epScrCnt)                   ; BC = # bytes to copy
        ld      hl,(epScrPtr)
        push    hl
        ld      de,-12
        add     hl,de
        ex      de,hl                           ; DE => destination
        pop     hl                              ; HL => source
        push    hl
        ldir
        pop     hl
        ld      de,gbuf+24
        CPHLDE()                                ; check if source has reached the 3rd top row of gbuf
        jr      z,afterEpilogueScrUpdate        ; if it has, no need to update the pointer/counter values
        ld      de,-12
        add     hl,de
        ld      (epScrPtr),hl                   ; update pointer for next frame
        ld      hl,(epScrCnt)
        ld      de,12
        add     hl,de
        ld      (epScrCnt),hl                   ; update counter for next frame
afterEpilogueScrUpdate:
        ; copy one row of videoBuffer to 2nd bottom row of gbuf
        ld      hl,videoBuffer
        ld      de,gbuf+768-24
        ld      bc,12
        ldir
        ; add left & right bordering pixels to the new row
        ld      hl,gbuf+768-24
        ld      a,(hl)
        or      %10000000
        ld      (hl),a
        ld      hl,gbuf+768-24+11
        ld      a,(hl)
        or      %00000001
        ld      (hl),a
        ; scroll videoBuffer up one row
        ld      hl,videoBuffer+12
        ld      de,videoBuffer
        ld      bc,6*12
        ldir
        bcall(_grbufcpy)
        ei
        ld      b,24
        call    shortDelay
        call    swapBuffers
        pop     bc
        djnz    epilogueScroll
        pop     hl
        ld      a,(hl)
        inc     a                               ; check if $FF terminator
        jp      nz,epilogueLoop
        ; scrolling is finished, now print the final screen
        inc     hl                              ; HL => page to print
        push    hl
        call    swapBuffers
        pop     hl
        jp      displayPageNoClear

;------------------------------------------------
; ep3EasterEgg - show the Episode III easter egg text
;   input:  none
;   output: none
;------------------------------------------------
ep3EasterEgg:
        ; show the text pages
        ld      hl,e3egg
        ld      b,(hl)                          ; B = # of pages
        inc     hl                              ; HL => start of text data
e3EasterEggLoop:
        push    bc
        push    hl
        call    textScreenBg
        pop     hl
        call    displayPageNoClear
        push    hl
        ld      hl,str2nd
        call    showStrings
        bcall(_grbufcpy)
        call    wait2ndEnter
        pop     hl
        pop     bc
        djnz    e3EasterEggLoop
        ret

.end
